@@ -233,7 +233,7 @@ bool Domain::event(QEvent* e) {
 
 	bool fromMaster = false;
 
-	switch (e->type()) {
+	switch (int(e->type())) {
 	case UPD_DM_MST_EV:
 		fromMaster = true;
 		// fall through
