@@ -101,6 +101,7 @@ protected slots:
 	void ActViewDiff_activated();
 	void ActViewDiffNewTab_activated();
 	void ActExternalDiff_activated();
+	void ActExternalEditor_activated();
 	void ActSplitView_activated();
 	void ActToggleLogsDiff_activated();
 	void ActShowDescHeader_activated();
@@ -113,9 +114,10 @@ protected slots:
 	void ActSettings_activated();
 	void ActCommit_activated();
 	void ActAmend_activated();
+	void ActCheckout_activated();
 	void ActBranch_activated();
 	void ActTag_activated();
-	void ActTagDelete_activated();
+	void ActDelete_activated();
 	void ActPush_activated();
 	void ActPop_activated();
 	void ActClose_activated();
@@ -128,6 +130,7 @@ protected slots:
 	void ActFilterTree_toggled(bool);
 	void ActAbout_activated();
 	void ActHelp_activated();
+	void ActMarkDiffToSha_activated();
 	void closeEvent(QCloseEvent* ce);
 
 private:
@@ -135,6 +138,7 @@ private:
 
 	virtual bool eventFilter(QObject* obj, QEvent* ev);
 	void updateGlobalActions(bool b);
+	void updateRevVariables(SCRef sha);
 	void setupShortcuts();
 	int currentTabType(Domain** t);
 	void filterList(bool isOn, bool onlyHighlight);
@@ -142,6 +146,7 @@ private:
 	void highlightAbbrevSha(SCRef abbrevSha);
 	void setRepository(SCRef wd, bool = false, bool = false, const QStringList* = NULL, bool = false);
 	void getExternalDiffArgs(QStringList* args, QStringList* filenames);
+	QStringList getExternalEditorArgs();
 	void lineEditSHASetText(SCRef text);
 	void updateCommitMenu(bool isStGITStack);
 	void updateRecentRepoMenu(SCRef newEntry = "");
@@ -167,15 +172,16 @@ private:
 	RevsView* rv;
 	QProgressBar* pbFileNamesLoading;
 
-	// curDir is the repository working dir, could be different from qgit running
+	// curDir is the repository working directory, could be different from qgit running
 	// directory QDir::current(). Note that qgit could be run from subdirectory
-	// so only after git->isArchive() that updates curDir to point to working dir
+	// so only after git->isArchive() that updates curDir to point to working directory
 	// we are sure is correct.
 	QString curDir;
 	QString startUpDir;
 	QString textToFind;
 	QRegExp shortLogRE;
 	QRegExp longLogRE;
+	QMap<QString, QVariant> revision_variables; // variables used in generic input dialogs
 	bool setRepositoryBusy;
 };
 
@@ -209,4 +215,24 @@ private:
 	}
 };
 
+class ExternalEditorProc : public QProcess {
+Q_OBJECT
+public:
+	ExternalEditorProc(QObject* p)
+	    : QProcess(p) {
+
+		connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
+		        this, SLOT(on_finished(int, QProcess::ExitStatus)));
+	}
+	~ExternalEditorProc() {
+		terminate();
+	}
+
+private slots:
+	void on_finished(int, QProcess::ExitStatus) { deleteLater(); }
+
+private:
+};
+
+
 #endif
