@@ -1,14 +1,13 @@
 /*
 	Description: interface to sync and async external program execution
 
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
 */
-#include <unistd.h> // usleep(), environ
-#include <qapplication.h>
-#include <qeventloop.h>
+#include <QApplication>
+#include <QTime>
 #include "exceptionmanager.h"
 #include "common.h"
 #include "domain.h"
@@ -19,178 +18,159 @@ MyProcess::MyProcess(QObject *go, Git* g, const QString& wd, bool err) : QProces
 	guiObject = go;
 	git = g;
 	workDir = wd;
-	receiver = NULL;
 	runOutput = NULL;
+	receiver = NULL;
 	errorReportingEnabled = err;
-	canceling = async = false;
-	exitStatus = true;
+	canceling = async = isWinShell = isErrorExit = false;
 }
 
-bool MyProcess::runAsync(SCRef rc, QObject* rcv, SCRef buf, QStringList* env) {
+bool MyProcess::runAsync(SCRef rc, QObject* rcv, SCRef buf) {
 
 	async = true;
 	runCmd = rc;
 	receiver = rcv;
 	setupSignals();
-	if (!launchMe(runCmd, buf, env))
+	if (!launchMe(runCmd, buf))
 		return false; // caller will delete us
 
 	return true;
 }
 
-bool MyProcess::runSync(SCRef rc, QByteArray* ro, QObject* rcv, SCRef buf, QStringList* env) {
+bool MyProcess::runSync(SCRef rc, QByteArray* ro, QObject* rcv, SCRef buf) {
 
 	async = false;
 	runCmd = rc;
 	runOutput = ro;
 	receiver = rcv;
 	if (runOutput)
-		runOutput->resize(0);
+		runOutput->clear();
 
 	setupSignals();
+	if (!launchMe(runCmd, buf))
+		return false;
+
+	QTime t;
+	t.start();
 
 	busy = true; // we have to wait here until we exit
 
-	if (!launchMe(runCmd, buf, env))
-		return false;
-
-	// we are now leaving our safe domain context and
-	// jump right in to processEvents() hyperspace
-	Domain* d = git->curContext();
-	git->setCurContext(NULL);
-
-	EM_BEFORE_PROCESS_EVENTS;
-
 	while (busy) {
-		/* we need a on_readyReadStdout() call here to avoid rare
-		 * hangs that could occur if launched program fullfills
-		 * its stdout buffer while we are sleeping
-		 */
-		on_readyReadStdout();
-		usleep(20000); // suspend 20ms to let OS reschedule
-		isRunning();
+		waitForFinished(20); // suspend 20ms to let OS reschedule
+
+		if (t.elapsed() > 200) {
+			EM_PROCESS_EVENTS;
+			t.restart();
+		}
 	}
-
-	EM_AFTER_PROCESS_EVENTS;
-
-	if (git->curContext())
-		qDebug("ASSERT in MyProcess::runSync, context is %p "
-		       "instead of NULL", (void*)git->curContext());
-
-	git->setCurContext(d); // restore our context
-
-	return exitStatus;
+	return !isErrorExit;
 }
 
 void MyProcess::setupSignals() {
 
-	connect(git, SIGNAL(cancelAllProcesses()), this, SLOT(on_cancel()));
-	connect(this, SIGNAL(readyReadStdout()), this, SLOT(on_readyReadStdout()));
-	connect(this, SIGNAL(processExited()), this, SLOT(on_processExited()));
+	connect(git, SIGNAL(cancelAllProcesses()),
+	        this, SLOT(on_cancel()));
+
+	connect(this, SIGNAL(readyReadStandardOutput()),
+	        this, SLOT(on_readyReadStandardOutput()));
+
+	connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
+	        this, SLOT(on_finished(int, QProcess::ExitStatus)));
+
 	if (receiver) {
-		connect(this, SIGNAL(readyReadStderr()), this, SLOT(on_readyReadStderr()));
+
+		connect(this, SIGNAL(readyReadStandardError ()),
+		        this, SLOT(on_readyReadStandardError()));
+
 		connect(this, SIGNAL(procDataReady(const QByteArray&)),
-		        receiver, SLOT(on_procDataReady(const QByteArray&)));
-		connect(this, SIGNAL(eof()), receiver, SLOT(on_eof()));
+		        receiver, SLOT(procReadyRead(const QByteArray&)));
+
+		connect(this, SIGNAL(eof()), receiver, SLOT(procFinished()));
 	}
 	Domain* d = git->curContext();
 	if (d)
 		connect(d, SIGNAL(cancelDomainProcesses()), this, SLOT(on_cancel()));
 }
 
-void MyProcess::sendErrorMsg(bool notStarted) {
+void MyProcess::sendErrorMsg(bool notStarted, SCRef err) {
 
 	if (!errorReportingEnabled)
 		return;
 
-	QString errorDesc(readStderr());
-	if (notStarted)
-		errorDesc = QString::fromAscii("Unable to start the process!");
+	QString errorDesc(readAllStandardError());
+	errorDesc.prepend(err);
 
-	const QString cmd(arguments().join(" ")); // hide any QUOTE_CHAR or related stuff
+	if (notStarted)
+		errorDesc = QString::fromLatin1("Unable to start the process!");
+
+	const QString cmd(arguments.join(" ")); // hide any QUOTE_CHAR or related stuff
 	MainExecErrorEvent* e = new MainExecErrorEvent(cmd, errorDesc);
 	QApplication::postEvent(guiObject, e);
 }
 
-void MyProcess::appendSystemEnvironment(QStringList* env) {
+bool MyProcess::launchMe(SCRef runCmd, SCRef buf) {
 
-	if (!env)
-		return;
+	arguments = splitArgList(runCmd);
+	if (arguments.isEmpty())
+		return false;
 
-	/* global 'environ' is an array of pointers to the
-	 * environment strings, defined in <unistd.h>
-	 */
-	char** envval = ::environ;
-	for ( ; *envval; envval++)
-		*env << QString(*envval);
-}
-
-bool MyProcess::launchMe(SCRef runCmd, SCRef buf, QStringList* env) {
-
-	const QStringList sl(splitArgList(runCmd));
-	setArguments(sl);
 	setWorkingDirectory(workDir);
-	appendSystemEnvironment(env);
-
-	connect(this, SIGNAL(launchFinished()), this, SLOT(on_launchFinished()));
-	isLaunching = true;
-
-	bool ok = launch(buf, env);
-	if (!ok)
+	if (!QGit::startProcess(this, arguments, buf, &isWinShell)) {
 		sendErrorMsg(true);
-
-	while (isLaunching)
-		EM_PROCESS_EVENTS;
-
-	return ok;
+		return false;
+	}
+	return true;
 }
 
-void MyProcess::on_launchFinished() {
-
-	disconnect(this, SIGNAL(launchFinished()), this, SLOT(on_launchFinished()));
-	isLaunching = false;
-}
-
-void MyProcess::on_readyReadStdout() {
-// workaround pipe buffer size limit. In case of big output, buffer
-// can became full and an hang occurs, so we read all data as soon
-// as possible and store it in runOutput
+void MyProcess::on_readyReadStandardOutput() {
 
 	if (canceling)
 		return;
 
 	if (receiver)
-		emit procDataReady(readStdout());
+		emit procDataReady(readAllStandardOutput());
 
 	else if (runOutput)
-		QGit::baAppend(*runOutput, readStdout());
-	else
-		readStdout(); // always flush input buffer
+		runOutput->append(readAllStandardOutput());
 }
 
-void MyProcess::on_readyReadStderr() {
+void MyProcess::on_readyReadStandardError() {
 
 	if (canceling)
 		return;
 
 	if (receiver)
-		emit procDataReady(readStderr()); // redirect to stdout
+		emit procDataReady(readAllStandardError()); // redirect to stdout
 	else
 		dbs("ASSERT in myReadFromStderr: NULL receiver");
 }
 
-void MyProcess::on_processExited() {
+void MyProcess::on_finished(int exitCode, QProcess::ExitStatus exitStatus) {
 
-	if (canceling || !normalExit() || canReadLineStderr())
-		exitStatus = false;
+	// Checking exingStatus is not reliable under Windows where if the
+	// process was terminated with TerminateProcess() from another
+	// application its value is still NormalExit
+	//
+	// Checking exit code for a failing command is unreliable too, as
+	// exmple 'git status' returns 1 also without errors.
+	//
+	// On Windows exit code seems reliable in case of a command wrapped
+	// in Window shell interpreter.
+	//
+	// So to detect a failing command we check also if stderr is not empty.
+	QString errorDesc(readAllStandardError());
+
+	isErrorExit =   (exitStatus != QProcess::NormalExit)
+	             || (exitCode != 0 && isWinShell)
+	             || !errorDesc.isEmpty()
+	             ||  canceling;
 
 	if (!canceling) { // no more noise after cancel
 
 		if (receiver)
 			emit eof();
 
-		if (!exitStatus)
-			sendErrorMsg();
+		if (isErrorExit)
+			sendErrorMsg(false, errorDesc);
 	}
 	busy = false;
 	if (async)
@@ -200,7 +180,13 @@ void MyProcess::on_processExited() {
 void MyProcess::on_cancel() {
 
 	canceling = true;
-	QProcess::tryTerminate();
+
+#ifdef Q_OS_WIN32
+	kill(); // uses TerminateProcess
+#else
+	terminate(); // uses SIGTERM signal
+#endif
+	waitForFinished();
 }
 
 const QStringList MyProcess::splitArgList(SCRef cmd) {
@@ -214,17 +200,17 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 	if (!(   cmd.contains(QGit::QUOTE_CHAR)
 	      || cmd.contains("\"")
 	      || cmd.contains("\'")))
-		return QStringList::split(' ', cmd);
+		return cmd.split(' ', QString::SkipEmptyParts);
 
 	// we have some work to do...
 	// first find a possible separator
 	const QString sepList("#%&!?"); // separator candidates
-	uint i = 0;
+	int i = 0;
 	while (cmd.contains(sepList[i]) && i < sepList.length())
 		i++;
 
 	if (i == sepList.length()) {
-		dbs("ASSERT no unique separator found");
+		dbs("ASSERT no unique separator found.");
 		return QStringList();
 	}
 	const QChar& sepChar(sepList[i]);
@@ -244,7 +230,7 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 
 	// QProcess::setArguments doesn't want quote
 	// delimited arguments, so remove trailing quotes
-	QStringList sl(QStringList::split(sepChar, newCmd));
+	QStringList sl(newCmd.split(sepChar, QString::SkipEmptyParts));
 	QStringList::iterator it(sl.begin());
 	for ( ; it != sl.end(); ++it) {
 		if (((*it).left(1) == "\"" && (*it).right(1) == "\"") ||
@@ -254,18 +240,18 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 	return sl;
 }
 
-void MyProcess::restoreSpaces(QString& newCmd, SCRef sepChar) {
+void MyProcess::restoreSpaces(QString& newCmd, const QChar& sepChar) {
 // restore spaces inside quoted text, supports nested quote types
 
-	QString quoteChar;
+	QChar quoteChar;
 	bool replace = false;
-	for (uint i = 0; i < newCmd.length(); i++) {
+	for (int i = 0; i < newCmd.length(); i++) {
 
 		const QChar& c = newCmd[i];
 
 		if (    !replace
-		    && (c == QGit::QUOTE_CHAR || c == "\"" || c == "\'")
-		    && (newCmd.contains(c) % 2 == 0)) {
+		    && (c == QGit::QUOTE_CHAR[0] || c == '\"' || c == '\'')
+		    && (newCmd.count(c) % 2 == 0)) {
 
 				replace = true;
 				quoteChar = c;
