@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,7 +7,7 @@
 #ifndef MYPROCESS_H
 #define MYPROCESS_H
 
-#include <qprocess.h>
+#include <QProcess>
 #include "git.h"
 
 class Git;
@@ -18,8 +18,8 @@ class MyProcess : public QProcess {
 Q_OBJECT
 public:
 	MyProcess(QObject *go, Git* g, const QString& wd, bool reportErrors);
-	bool runSync(SCRef runCmd, QByteArray* runOutput, QObject* rcv, SCRef buf, QStringList* env);
-	bool runAsync(SCRef rc, QObject* rcv, SCRef buf, QStringList* env);
+	bool runSync(SCRef runCmd, QByteArray* runOutput, QObject* rcv, SCRef buf);
+	bool runAsync(SCRef rc, QObject* rcv, SCRef buf);
 	static const QStringList splitArgList(SCRef cmd);
 
 signals:
@@ -30,17 +30,15 @@ public slots:
 	void on_cancel();
 
 private slots:
-	void on_readyReadStdout();
-	void on_readyReadStderr();
-	void on_processExited();
-	void on_launchFinished();
+	void on_readyReadStandardOutput();
+	void on_readyReadStandardError();
+	void on_finished(int, QProcess::ExitStatus);
 
 private:
 	void setupSignals();
-	void appendSystemEnvironment(QStringList* env);
-	bool launchMe(SCRef runCmd, SCRef buf, QStringList* env);
-	void sendErrorMsg(bool notStarted = false);
-	static void restoreSpaces(QString& newCmd, SCRef sepChar);
+	bool launchMe(SCRef runCmd, SCRef buf);
+	void sendErrorMsg(bool notStarted = false, SCRef errDesc = "");
+	static void restoreSpaces(QString& newCmd, const QChar& sepChar);
 
 	QObject* guiObject;
 	Git* git;
@@ -48,12 +46,13 @@ private:
 	QByteArray* runOutput;
 	QString workDir;
 	QObject* receiver;
+	QStringList arguments;
 	bool errorReportingEnabled;
 	bool canceling;
 	bool busy;
-	bool exitStatus;
 	bool async;
-	bool isLaunching;
+	bool isWinShell;
+	bool isErrorExit;
 };
 
 #endif
