@@ -278,8 +278,11 @@ bool RevsView::doUpdate(bool force) {
 			bool isDir = m()->treeView->isDir(st.fileName());
 			m()->updateContextActions(st.sha(), st.fileName(), isDir, found);
 		}
-		if (st.isChanged() || force)
+		if (st.isChanged() || force) {
+			// activate log or diff tab depending on file selection
+			tab()->tabLogDiff->setCurrentIndex(st.fileName().isEmpty() ? 0 : 1);
 			tab()->textEditDiff->centerOnFileHeader(st);
+		}
 
 		// at the end update diffs that is the slowest and must be
 		// run after update of file list for 'diff to sha' to work
@@ -313,10 +316,10 @@ void RevsView::updateLineEditSHA(bool clear) {
 	m()->ActForward->setEnabled(l->isRedoAvailable());
 }
 
-void RevsView::on_lanesContextMenuRequested(SCList parents, SCList childs) {
+void RevsView::on_lanesContextMenuRequested(SCList parents, SCList children) {
 
 	QMenu contextMenu;
-	FOREACH_SL (it, childs)
+	FOREACH_SL (it, children)
 		contextMenu.addAction("Child: " + git->getShortLog(*it));
 
 	FOREACH_SL (it, parents) {
@@ -327,9 +330,9 @@ void RevsView::on_lanesContextMenuRequested(SCList parents, SCList childs) {
 	if (!act)
 		return;
 
-	int cc = childs.count();
+	int cc = children.count();
 	int id = contextMenu.actions().indexOf(act);
-	SCRef target(id < cc ? childs[id] : parents[id - cc]);
+	SCRef target(id < cc ? children[id] : parents[id - cc]);
 	st.setSha(target);
 	UPDATE();
 }
